<?php
date_default_timezone_set("Asia/Tokyo");
$msg = null;

//sqlite
$db_file = __DIR__.DIRECTORY_SEPARATOR."yubin.db";

//update
if(isset($_POST["japanpost-zip"])){
    set_time_limit(0);
    $url = $_POST["japanpost-zip"];
    $zip_file = __DIR__.DIRECTORY_SEPARATOR.date("Ymd_his").".zip";
    $csv_file = __DIR__.DIRECTORY_SEPARATOR.date("Ymd_his").".csv";
    download_zip($url, $zip_file);
    zip_2_csv($zip_file, $csv_file);
    csv_2_db($csv_file, $db_file);
    unlink($csv_file);
    $msg = "更新しました。";
}

//db version
$ver = db_version($db_file);
if($ver == '---'){ $msg = "データベースの準備ができていません。「更新」ボタンをクリックしてください。"; }

/** データベースファイルから、最終更新日時を取得します. */
function db_version($db_file){
    if(!file_exists($db_file)){ return "---";  }
    $pdo = new \PDO("sqlite:".$db_file);
    $pdo->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);
    $stmt = $pdo->prepare("SELECT * FROM ver WHERE id = (SELECT MAX(id) FROM ver);");
    $stmt->execute();
    $stmt->setFetchMode(\PDO::FETCH_ASSOC);
    $row = $stmt->fetch();
    if(empty($row)){ return "---"; }
    return $row["date"];
}

/** URLから、郵便番号データZIPをダウンロードします. */
function download_zip($url, $zip_file){
    $ch = curl_init($url);
    try{
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_USERAGENT, "curl");
        curl_setopt($ch, CURLOPT_POST, TRUE); 
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $file = curl_exec($ch);
        curl_close($ch);
        file_put_contents($zip_file, $file);
    } catch (Exception $ex) {
        curl_close($ch);
        echo htmlspecialchars("ERROR: ".$ex->getMessage());
        die();
    }
}
/** 郵便番号データZIPから、CSVファイルを取り出します. */
function zip_2_csv($zip_file, $csv_file){
    $zip = new \ZipArchive();
    if(!$zip->open($zip_file)){
        echo htmlspecialchars("ERROR: ZIPファイル [{$zip_file}] を開けません。");
        die();
    }
    $fs = $zip->getStream($zip->getNameIndex(0));
    $content = null;
    while (!feof($fs)){ $content.= fread($fs, 2);}
    file_put_contents($csv_file,  mb_convert_encoding($content,"utf-8","sjis"));
    fclose($fs);
    $zip->close();
}
/** 郵便番号データCSVファイルから、SQLITEのデータベースを構築・登録します. */
function csv_2_db($csv_file, $db_file){
    $csql = "CREATE TABLE IF NOT EXISTS dat("
            . "jisx TEXT, zipcode_old TEXT, zipcode TEXT, addr1_kana TEXT, addr2_kana TEXT, addr3_kana TEXT, addr1 TEXT, addr2 TEXT, addr3 TEXT, "
            . "flag10 INTEGER, flag11 INTEGER, flag12 INTEGER, flag13 INTEGER, flag14 INTEGER, flag15 INTEGER"
            . ");";
    $csql.= "CREATE TABLE IF NOT EXISTS ver(id INTEGER PRIMARY KEY AUTOINCREMENT, date TEXT, rows INTEGER);";
    $csql.= "CREATE INDEX IF NOT EXISTS idx_zipcode on dat(zipcode);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag10 on dat(flag10);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag11 on dat(flag11);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag12 on dat(flag12);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag13 on dat(flag13);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag14 on dat(flag14);";
    //$csql.= "CREATE INDEX IF NOT EXISTS idx_flag15 on dat(flag15);";
    
    $pdo = new \PDO("sqlite:".$db_file);
    $pdo->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);
    $pdo->exec($csql);
    
    $f = new \SplFileObject($csv_file, "r");
    $f->setFlags(\SplFileObject::READ_CSV | SplFileObject::SKIP_EMPTY);
    $pdo->beginTransaction();
    
    $idx_code = 2;
    $idx_addr3_kana = 5;
    $idx_addr3 = 8;
    $idx_flag13 = 14;
    
    try{
        $rows = 0;
        $pre_code = null;
        $pdo->exec("DELETE FROM dat;");
        $stmt = $pdo->prepare("INSERT INTO dat VALUES(?,?,?,?,?,?,?,?,?, ?,?,?,?,?,?);");
        
        foreach($f as $cols){
            if(empty($cols)){continue;}
            //連続して同一の郵便番号、かつflag13が"0"のものをスキップ
            if($cols[$idx_code] === $pre_code && $cols[$idx_flag13] == "0"){continue;}
            
            $pre_code = $cols[$idx_code];
            $addr3 = $cols[$idx_addr3];
            $kana3 = $cols[$idx_addr3_kana];
            
            //特定の文字である場合は、ブランクへ
            if($addr3 === "以下に掲載がない場合" ||$addr3 === "境町の次に番地がくる場合"){
                $addr3 = "";
                $kana3 = "";
            }
            
            //「一円」を含む場合かつ、「一円」単体ではない場合ブランク
            if(strpos($addr3, "一円") && $addr3 !== "一円"){
                $addr3 = "";
                $kana3 = "";
            }
            
            //"、"を含む場合は、ブランクへ
            if(strpos($addr3, "、")){
                $addr3 = "";
                $kana3 = "";
            }
            
            //"（）"より前の文字だけを取得
            $kana3 = explode("(", $kana3)[0];
            $addr3 = explode("（", $addr3)[0];
            
            //データ登録
            $cols[$idx_addr3] = $addr3;
            $cols[$idx_addr3_kana] = $kana3;
            $stmt->execute($cols);
            $rows++;
        }
        $stamp = date("Y-m-d H:i:s");
        $pdo->exec("INSERT INTO ver(date,rows) VALUES('{$stamp}',{$rows});");
        $pdo->commit();
    } catch (Exception $ex) {
        $pdo->rollBack();
        echo htmlspecialchars("ERROR: ".$ex->getMessage());
        die();
    }
}
?>
<!DOCTYPE html>
<html lang="ja">
<head><title>ローカル郵便番号API</title><meta charset="UTF-8"></head>
<body>
    <h2>データベース</h2>
    <div style="color:#f00;"><?php echo htmlspecialchars($msg); ?></div>
    <form method="post" action="./index.php">
        <p>DBバージョン: <span style="color: #f00;"><?= htmlspecialchars($ver); ?></span></p>
        <input type="text" name="japanpost-zip" value="https://www.post.japanpost.jp/zipcode/dl/kogaki/zip/ken_all.zip" style="width: 50vw;">
        <button type="submit">更新</button>
    </form>
    
    <ul>
        <li><a href="https://www.post.japanpost.jp/zipcode/dl/kogaki-zip.html" target="_blank">日本郵便のサイト</a>より更新日を確認してください。</li>
        <li>現在のDBバージョンよりも新しい場合は、「更新」ボタンをクリックしてください。</li>
        <li>DBバージョンに日時が表示されていれば完了です。</li>
    </ul>

    <hr>
    
    <h2>動作確認</h2>
    <span id="api-url">./api.php?zipcode=</span><input type="text" id="zipcode" maxlength="8" placeholder="000-0000">
    <button type="button" id="call-api">検索</button>
    <div style="background-color: #eee; padding: 1rem; margin-top: 1rem;">
        <input type="text" id="result-addr1">
        <input type="text" id="result-addr2">
        <input type="text" id="result-addr3">
        <div id="result-msg" style="color:#f00;"></div>
    </div>
    <ul>
        <li>アドレスバー末尾の入力欄へ任意の郵便番号を入力してください。</li>
        <li>「検索」ボタンをクリックしてください。</li>
        <li>グレー枠内の3項目に住所データが表示されていれば完了です。</li>
    </ul>
</body>

<script src="https://ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min.js"></script>
<script>
    let apiUrl = document.createElement('a');
    apiUrl.href = './api.php?zipcode=';
    $('#api-url').text(apiUrl.href);
    
    $('#call-api').click(function(){
        callApi($("#zipcode").val());
    });
    
    function callApi(zipcode){
        if(zipcode.length < 7 || zipcode.length > 8){
            setResult('length-error: ','','','');
            return;
        }
        
        $.ajax({
            type: 'get',
            url: apiUrl.href + zipcode,
            dataType: 'json',
            timeout: 1000
        }).done(function(json) {
            if(json.find){
                var data = json.data[0];
                setResult('ok:', data.addr1, data.addr2, data.addr3);
            }else{
                setResult('not-found:','','','');
            }
        }).fail(function(){
            setResult('error: ','','','');
        });
    }
    function setResult(msg, addr1, addr2, addr3){
        $('#result-msg').text(msg);
        $('#result-addr1').val(addr1);
        $('#result-addr2').val(addr2);
        $('#result-addr3').val(addr3);
    }
</script>
</html>